package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import org.apache.commons.lang.StringUtils;
import java.net.http.HttpResponse;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.bpcl.lpg.facades.data.LpgRequestData;
import com.bpcl.lpg.core.pojo.RspResponseData;
import java.util.Objects;
import com.bpcl.core.model.BPCLCustomerModel;
import java.util.List;
import java.text.MessageFormat;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import java.util.Collection;
import org.apache.commons.collections4.CollectionUtils;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import com.bpcl.lpg.core.service.LpgConnectionService;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import com.bpcl.lpg.core.encryption.LpgEncryptionService;
import com.bpcl.core.customer.dao.BpclCustomerDao;
import org.slf4j.Logger;
import de.hybris.platform.cronjob.model.CronJobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class SetCustomerConsumerInfoJob extends AbstractJobPerformable<CronJobModel>
{
    private static final Logger LOG;
    private BpclCustomerDao bpclCustomerDao;
    private LpgEncryptionService lpgEncryptionService;
    private LPGCredentialsService lpgCredentialsService;
    private LpgConnectionService rspConnectionService;
    private NotificationEngineService notificationEngineService;
    private ConfigurationService configurationService;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    private static final String DISTRIBUTOR_DETAILS = "13";
    private static final String ZERO = "0";
    
    public PerformResult perform(final CronJobModel cronJobModel) {
        try {
            final List<BPCLCustomerModel> customers = this.bpclCustomerDao.getAllCustomersWithoutDistId();
            if (CollectionUtils.isNotEmpty((Collection)customers)) {
                SetCustomerConsumerInfoJob.LOG.info("No of Customers to update : {}", (Object)customers.size());
                this.populateConsumerInfo(customers);
            }
            return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
        }
        catch (final Exception ex) {
            SetCustomerConsumerInfoJob.LOG.error(ex.getMessage(), ex.getCause());
            final String body = MessageFormat.format("Set customer consumer info cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Set Customer Consumer Info Job Alert", body, this.configurationService.getConfiguration().getString("lpg.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
    }
    
    private void populateConsumerInfo(final List<BPCLCustomerModel> customers) {
        if (CollectionUtils.isNotEmpty((Collection)customers)) {
            customers.forEach(c -> {
                final RspResponseData rspResponseData = this.getRspDetails(c);
                final String customer = c.getCustomerID();
                if (Objects.nonNull(rspResponseData)) {
                    SetCustomerConsumerInfoJob.LOG.info("Processing customer {}", customer);
                    this.modelService.save((Object)c);
                    SetCustomerConsumerInfoJob.LOG.info("Consumer details updated for customer {}", customer);
                }
                else {
                    SetCustomerConsumerInfoJob.LOG.error("RSP response is null for customer {}", customer);
                }
            });
        }
    }
    
    private RspResponseData getRspDetails(final BPCLCustomerModel c) {
        final LpgRequestData requestData = new LpgRequestData();
        requestData.setContactNumber(c.getContactNumber());
        requestData.setUniqueAPIReqNo("13");
        requestData.setRequestType("13");
        requestData.setEndPointName("distDetailsUrl");
        this.getDefaultValuesForNonMandatoryFields(requestData);
        return this.rspDetails(requestData);
    }
    
    public RspResponseData rspDetails(final LpgRequestData requestData) {
        this.populateBasicDetails(requestData);
        HttpResponse<String> httpResponse = null;
        RspResponseData rspResponseData = null;
        try {
            httpResponse = this.getRspConnectionService().getResponse(requestData, "", requestData.getEndPointName());
            if (Objects.nonNull(httpResponse)) {
                final ObjectMapper objectMapper = new ObjectMapper();
                objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
                rspResponseData = objectMapper.readValue(httpResponse.body(), RspResponseData.class);
                this.generateConsumerId(rspResponseData);
            }
        }
        catch (final JsonProcessingException e) {
            SetCustomerConsumerInfoJob.LOG.error("Exception while getting RSP Data for customer {}", requestData.getContactNumber());
        }
        return rspResponseData;
    }
    
    private void populateBasicDetails(final LpgRequestData requestData) {
        this.populateDefaultContactNoTypeIfBlank(requestData);
        final String hashValue = this.getHashKeyValueRequest(requestData);
        requestData.setHashKey(hashValue);
        requestData.setHashValue(hashValue);
    }
    
    private void populateDefaultContactNoTypeIfBlank(final LpgRequestData requestData) {
        final String contactNumberType = requestData.getContactNumberType();
        if (StringUtils.isBlank(contactNumberType)) {
            requestData.setContactNumberType("M");
        }
    }
    
    private String getHashKeyValueRequest(final LpgRequestData requestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = requestData.getEndPointName();
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        requestData.setPartnerCode(partnerCode);
        hashMessage.append(this.getEmptyIfBlank(partnerCode));
        hashMessage.append(this.getEmptyIfBlank(requestData.getPartnerTransId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getUniqueAPIReqNo()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumberType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumber()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getLpgId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getConsumerNo()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getServiceRequestType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getServiceReqID()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getServiceCategory()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getMechanicServiceType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getRefillType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getProductType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getProductQty()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getBookingRequestTime()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getRefillId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl1()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl2()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl3()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl4()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getAddl5()));
        hashMessage.append(this.getEmptyIfBlank(secret));
        final String encryptedMsg = this.getLpgEncryptionService().encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    private String getEmptyIfBlank(final String stringToCheck) {
        return StringUtils.isBlank(stringToCheck) ? "" : stringToCheck;
    }
    
    private void generateConsumerId(final RspResponseData rspResponseData) {
        if (null != rspResponseData && StringUtils.isNotEmpty(rspResponseData.getLpgId())) {
            final String lpgId = rspResponseData.getLpgId();
            final String id = "1".concat(StringUtils.repeat("0", lpgId.length() - 1));
            final String consumerId = String.valueOf(Long.parseLong(lpgId) - Long.parseLong(id));
            rspResponseData.setConsumerId(consumerId);
        }
    }
    
    private void getDefaultValuesForNonMandatoryFields(final LpgRequestData data) {
        if (StringUtils.isBlank(data.getContactNumberType())) {
            data.setContactNumberType("");
        }
        if (StringUtils.isBlank(data.getRefillType())) {
            data.setRefillType("");
        }
        if (StringUtils.isBlank(data.getRefillId())) {
            data.setRefillId("");
        }
        if (StringUtils.isBlank(data.getProductType())) {
            data.setProductType("");
        }
        if (StringUtils.isBlank(data.getProductQty())) {
            data.setProductQty("");
        }
        if (StringUtils.isBlank(data.getBookingRequestTime())) {
            data.setBookingRequestTime("");
        }
        if (StringUtils.isBlank(data.getUniqueAPIReqNo())) {
            data.setUniqueAPIReqNo("");
        }
        if (StringUtils.isBlank(data.getRequestType())) {
            data.setRequestType("");
        }
        if (StringUtils.isBlank(data.getContactNumber())) {
            data.setContactNumber("");
        }
        if (StringUtils.isBlank(data.getPartnerTransId())) {
            data.setPartnerTransId("");
        }
        if (StringUtils.isBlank(data.getConsumerNo())) {
            data.setConsumerNo("");
        }
        if (StringUtils.isBlank(data.getDistCode())) {
            data.setDistCode("");
        }
        if (StringUtils.isBlank(data.getLpgId())) {
            data.setLpgId("");
        }
        if (StringUtils.isBlank(data.getRequestId())) {
            data.setRequestId("");
        }
        if (StringUtils.isBlank(data.getApiRequestNo())) {
            data.setApiRequestNo("");
        }
        if (StringUtils.isBlank(data.getStateCode())) {
            data.setStateCode("");
        }
        if (StringUtils.isBlank(data.getDistrictCode())) {
            data.setDistrictCode("");
        }
        if (StringUtils.isBlank(data.getIfscCode())) {
            data.setIfscCode("");
        }
        if (StringUtils.isBlank(data.getTypeOfDedup())) {
            data.setTypeOfDedup("");
        }
        if (StringUtils.isBlank(data.getMobileNo())) {
            data.setMobileNo("");
        }
        if (StringUtils.isBlank(data.getAhlTin())) {
            data.setAhlTin("");
        }
        if (StringUtils.isBlank(data.getBankIFSC())) {
            data.setBankIFSC("");
        }
        if (StringUtils.isBlank(data.getHouseRationCardNo())) {
            data.setHouseRationCardNo("");
        }
        if (StringUtils.isBlank(data.getConsumerMiddleName())) {
            data.setConsumerMiddleName("");
        }
        if (StringUtils.isBlank(data.getAddl1())) {
            data.setAddl1("");
        }
        if (StringUtils.isBlank(data.getAddl2())) {
            data.setAddl2("");
        }
        if (StringUtils.isBlank(data.getAddl3())) {
            data.setAddl3("");
        }
        if (StringUtils.isBlank(data.getSpouseName())) {
            data.setSpouseName("");
        }
        if (StringUtils.isBlank(data.getFatherName())) {
            data.setFatherName("");
        }
        if (StringUtils.isBlank(data.getVillageName())) {
            data.setVillageName("");
        }
        if (StringUtils.isBlank(data.getSubDistrictName())) {
            data.setSubDistrictName("");
        }
        if (StringUtils.isBlank(data.getAadhaarNo())) {
            data.setAadhaarNo("");
        }
        if (StringUtils.isBlank(data.getBankName())) {
            data.setBankName("");
        }
        if (StringUtils.isBlank(data.getBankIFSCCode())) {
            data.setBankIFSCCode("");
        }
        if (StringUtils.isBlank(data.getRegistrationRefNo())) {
            data.setRegistrationRefNo("");
        }
        if (StringUtils.isBlank(data.getBankAccountNo())) {
            data.setBankAccountNo("");
        }
        if (StringUtils.isBlank(data.getNameInBankAccount())) {
            data.setNameInBankAccount("");
        }
        if (StringUtils.isBlank(data.getRationCardNo())) {
            data.setRationCardNo("");
        }
        if (StringUtils.isBlank(data.getCylinderType())) {
            data.setCylinderType("");
        }
        if (StringUtils.isBlank(data.getEmailId())) {
            data.setEmailId("");
        }
        if (StringUtils.isBlank(data.getVillageCode())) {
            data.setVillageCode("0");
        }
        if (StringUtils.isBlank(data.getSubDistrictCode())) {
            data.setSubDistrictCode("0");
        }
        if (StringUtils.isBlank(data.getAccountType())) {
            data.setAccountType("0");
        }
        if (StringUtils.isBlank(data.getRationCardStateCd())) {
            data.setRationCardStateCd("");
        }
        if (StringUtils.isBlank(data.getMechanicServiceType())) {
            data.setMechanicServiceType("");
        }
        if (StringUtils.isBlank(data.getServiceCategory())) {
            data.setServiceCategory("");
        }
        if (StringUtils.isBlank(data.getAddl3())) {
            data.setAddl3("");
        }
        if (StringUtils.isBlank(data.getAddl4())) {
            data.setAddl4("");
        }
        if (StringUtils.isBlank(data.getAddl5())) {
            data.setAddl5("");
        }
        if (StringUtils.isBlank(data.getServiceRequestType())) {
            data.setServiceRequestType("");
        }
        if (StringUtils.isBlank(data.getServiceReqID())) {
            data.setServiceReqID("");
        }
        if (StringUtils.isBlank(data.getDistributorSearchType())) {
            data.setDistributorSearchType("");
        }
        if (StringUtils.isBlank(data.getPinCode())) {
            data.setPinCode("");
        }
        if (StringUtils.isBlank(data.getLatitude())) {
            data.setLatitude("");
        }
        if (StringUtils.isBlank(data.getLongitude())) {
            data.setLongitude("");
        }
        if (StringUtils.isBlank(data.getDestinationDistributorCode())) {
            data.setDestinationDistributorCode("");
        }
        if (StringUtils.isBlank(data.getDestinationDistributorOmc())) {
            data.setDestinationDistributorOmc("");
        }
        if (StringUtils.isBlank(data.getPortabilityReason())) {
            data.setPortabilityReason("");
        }
    }
    
    public boolean isAbortable() {
        return Boolean.TRUE;
    }
    
    public void setBpclCustomerDao(final BpclCustomerDao bpclCustomerDao) {
        this.bpclCustomerDao = bpclCustomerDao;
    }
    
    public LpgEncryptionService getLpgEncryptionService() {
        return this.lpgEncryptionService;
    }
    
    public void setLpgEncryptionService(final LpgEncryptionService lpgEncryptionService) {
        this.lpgEncryptionService = lpgEncryptionService;
    }
    
    public void setLpgCredentialsService(final LPGCredentialsService lpgCredentialsService) {
        this.lpgCredentialsService = lpgCredentialsService;
    }
    
    public LpgConnectionService getRspConnectionService() {
        return this.rspConnectionService;
    }
    
    public void setRspConnectionService(final LpgConnectionService rspConnectionService) {
        this.rspConnectionService = rspConnectionService;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(SetCustomerConsumerInfoJob.class);
    }
}
